#ifndef BD_STRING
#define BD_STRING

#include <iostream.h>

class String
{
private:
	char *cString;
	unsigned char pascalString[256];
	char shiftFiller;
	
public:
	String();
	//Default constructor
	
	String(const String &copyMe);
	//Copy constructor
	
	String(const char copyMe[]);
	//Constructor
	
	String(const unsigned char copyMe[]);
	//Constructor (for Pascal strings)
	
	String(char copyMe);
	//Constructor
	
//	String(unsigned char copyMe);
	//Constructor
	
	String(short copyMe);
	//Constructor
	
	String(long copyMe);
	//Constructor
	
	String(float copyMe);
	//Constructor
	
	String(unsigned short copyMe);
	//Constructor
	
	String(unsigned long copyMe);
	//Constructor
	
	String(bool copyMe);
	//Constructor
	
	virtual ~String();
	//Destructor
	
	void setShiftFiller(char ShiftFiller);
	//Sets the filler for << and >> operation
	//Note that passing NULL will result in strings be chopped off when shifted
	
	char getShiftFiller() const;
	//Gets the filler used in << and >> operations
	
	const char *toCString() const;
	//Converts it to a C string
	
	const unsigned char *toPascalString() const;
	//Converts it to a Pascal string
	
	char toChar() const;
	//Converts it to a char
	//(It just returns NULL if it is empty or it returns the first character if the string)
	
//	char toUnsignedChar() const;
	//Converts it to a char
	//(It just returns NULL if it is empty or it returns the first character if the string)
	
	short toShort() const;
	//Converts it to a short
	
	long toLong() const;
	//Converts it to a long
	
	float toFloat() const;
	//Converts it to a float
	
	unsigned short toUnsignedShort() const;
	//Converts it to an unsigned short
	
	unsigned long toUnsignedLong() const;
	//Converts it to an unsigned long
	
	bool toBool() const;
	//Converts it to a bool
	
	const String toUpper() const;
	//Converts to uppercase
	
	const String toLower() const;
	//Converts to lowercase
	
	const String stripLeading() const;
	//Strips the leading white space
	
	const String stripTrailing() const;
	//Strips the trailing white space
	
	unsigned long getLength() const;
	//Get's its length
	
	const String left(unsigned long len) const;
	//Gets the left len characters
	
	const String right(unsigned long len) const;
	//Gets the right len characters
	
	const String mid(unsigned long offest, unsigned long len) const;
	//Gets the middle len characters at the base-zero offset
	
	const String mid(unsigned long offest) const;
	//Gets the middle len characters from the base-zero offset to the end of the string
	
	long find(const String &findMe) const;
	//Returns the 0-based position of the substring findMe or -1 if it isn't found
	
	long find(const String &findMe, unsigned long offset) const;
	//Returns the 0-based position of the substring findMe or -1 if it isn't found
	//Starts the search at the 0-based position specified by offset
	
	bool isAlpha() const;
	//Returns true if the character are all letters; returns false otherwise
	
	bool isNumber() const;
	//Returns true if the the string is a number
	//Note that it will ignore leading and trailing white space
	//Note that it will check for negative signs and decimal points and verify that they are in the right spot
	
	const String &getLine(istream &inputStream, unsigned long maxChar);
	//Does inputStream.getline(#this#, maxChar);
	
	const String &readRaw(istream &inputStream, unsigned long maxChar);
	//Does inputStream.read(#this#, maxChar);
	
	unsigned long getTokenCount() const;
	//Returns the number of white space deliniated tokens
	
	unsigned long getTokenCount(const String &delimiters) const;
	//Returns the number of tokens delimited by the characters in delimiters
	
	String getToken(unsigned long offset) const;
	//Returns the white space deliniated token at the passed offset
	
	String getToken(unsigned long offset, const String &delimiters) const;
	//Returns the token at the passed offset delimited by the characters in delimiters
	
	String getTokens(unsigned long offset, unsigned long amount) const;
	//Returns amount white space deliniated tokens at the passed offset; if amount==0 then it returns all remaining tokens after offset instead of amount tokens
	
	String getTokens(unsigned long offset, unsigned long amount, const String &delimiters) const;
	//Returns amount tokens at the passed offset deliniated by the characters in delimiters; if amount==0 then it returns all remaining tokens after offset instead of amount tokens
	
	String flattenTokens();
	//Removes all white space so you get one flat token
	
	String flattenTokens(const String &delimiters);
	//Removes all passed token deliniators so you get one flat token
	
private:
	void setPascalString();
	//Sets the Pascal string variable
	
public:
	
	//Operators...
	
	String operator<<(long offset) const;
	//<< operator
	//Shifts the characters to the left and fills the end with the shift filler character
	
	String operator>>(long offset) const;
	//>> operator
	//Shifts the characters to the right and fills the end with the shift filler character
	
	
	//Text
	
	virtual const String &operator=(const String &assignMe);
	//= operator
	
	virtual const String &operator=(const char assignMe[]);
	//= operator
	
	virtual const String &operator=(const unsigned char assignMe[]);
	//= operator (for Pascal strings)
	
	virtual const String &operator=(char assignMe);
	//= operator
	
//	virtual const String &operator=(unsigned char assignMe);
	//= operator
	
	bool operator==(const String &contrastMe) const;
	//== operator
	
	bool operator==(const char contrastMe[]) const;
	//== operator
	
	bool operator==(const unsigned char contrastMe[]) const;
	//== operator (for Pascal strings)
	
	bool operator==(char contrastMe) const;
	//== operator
	
//	bool operator==(unsigned char contrastMe) const;
	//== operator
	
	bool operator!=(const String &contrastMe) const;
	//!= operator
	
	bool operator!=(const char contrastMe[]) const;
	//!= operator
	
	bool operator!=(const unsigned char contrastMe[]) const;
	//!= operator (for Pascal strings)
	
	bool operator!=(char contrastMe) const;
	//!= operator
	
//	bool operator!=(unsigned char contrastMe) const;
	//!= operator
	
	bool operator>(const String &contrastMe) const;
	//> operator
	
	bool operator>(const char contrastMe[]) const;
	//> operator
	
	bool operator>(const unsigned char contrastMe[]) const;
	//> operator (for Pascal strings)
	
	bool operator>(char contrastMe) const;
	//> operator
	
//	bool operator>(unsigned char contrastMe) const;
	//> operator
	
	bool operator<(const String &contrastMe) const;
	//< operator
	
	bool operator<(const char contrastMe[]) const;
	//< operator
	
	bool operator<(const unsigned char contrastMe[]) const;
	//< operator (for Pascal strings)
	
	bool operator<(char contrastMe) const;
	//< operator
	
//	bool operator<(unsigned char contrastMe) const;
	//< operator
	
	bool operator>=(const String &contrastMe) const;
	//>= operator
	
	bool operator>=(const unsigned char contrastMe[]) const;
	//>= operator (for Pascal strings)
	
	bool operator>=(const char contrastMe[]) const;
	//>= operator
	
	bool operator>=(char contrastMe) const;
	//>= operator
	
//	bool operator>=(unsigned char contrastMe) const;
	//>= operator
	
	bool operator<=(const String &contrastMe) const;
	//<= operator
	
	bool operator<=(const char contrastMe[]) const;
	//<= operator
	
	bool operator<=(const unsigned char contrastMe[]) const;
	//<= operator (for Pascal strings)
	
	bool operator<=(char contrastMe) const;
	//<= operator
	
//	bool operator<=(unsigned char contrastMe) const;
	//<= operator
	
	virtual const String &operator+=(const String &appendMe);
	//=+ operator
	
	virtual const String &operator+=(const char appendMe[]);
	//=+ operator
	
	virtual const String &operator+=(const unsigned char appendMe[]);
	//=+ operator (for Pascal strings)
	
	virtual const String &operator+=(char appendMe);
	//=+ operator
	
//	virtual const String &operator+=(unsigned char appendMe);
	//=+ operator
	
	const String operator+(const String &appendMe) const;
	//+ operator
	
	const String operator+(const char appendMe[]) const;
	//+operator
	
	const String operator+(const unsigned char appendMe[]) const;
	//+operator (for Pascal strings)
	
	const String operator+(char appendMe) const;
	//+operator
	
//	const String operator+(unsigned char appendMe) const;
	//+operator
	
	
	//Numeric
	
	virtual const String &operator=(short assignMe);
	//= operator
	
	virtual const String &operator=(long assignMe);
	//= operator
	
	virtual const String &operator=(float assignMe);
	//= operator
	
	virtual const String &operator=(unsigned short assignMe);
	//= operator
	
	virtual const String &operator=(unsigned long assignMe);
	//= operator
	
	
	//Boolean
	
	virtual const String &operator=(bool assignMe);
	//= operator
	
	
	//Other
	virtual const String operator[](unsigned long offset) const;
};


//Non-member operators

bool operator==(const char charArrayParam[], const String &stringParam);
//== operator

bool operator==(const unsigned char charArrayParam[], const String &stringParam);
//== operator (for Pascal strings)

bool operator==(char charParam, const String &stringParam);
//== operator

//bool operator==(unsigned char charParam, const String &stringParam);
//== operator

bool operator!=(const char charArrayParam[], const String &stringParam);
//!= operator

bool operator!=(const unsigned char charArrayParam[], const String &stringParam);
//!= operator (for Pascal strings)

bool operator!=(char charParam, const String &stringParam);
//!= operator

//bool operator!=(unsigned char charParam, const String &stringParam);
//!= operator

bool operator>(const char charArrayParam[], const String &stringParam);
//> operator

bool operator>(const unsigned char charArrayParam[], const String &stringParam);
//> operator (for Pascal strings)

bool operator>(char charParam, const String &stringParam);
//> operator

//bool operator>(unsigned char charParam, const String &stringParam);
//> operator

bool operator<(const char charArrayParam[], const String &stringParam);
//< operator

bool operator<(const unsigned char charArrayParam[], const String &stringParam);
//< operator (for Pascal strings)

bool operator<(char charParam, const String &stringParam);
//< operator

//bool operator<(unsigned char charParam, const String &stringParam);
//< operator

bool operator>=(const char charArrayParam[], const String &stringParam);
//>= operator

bool operator>=(const unsigned char charArrayParam[], const String &stringParam);
//>= operator (for Pascal strings)

bool operator>=(char charParam, const String &stringParam) ;
//>= operator

//bool operator>=(unsigned char charParam, const String &stringParam) ;
//>= operator

bool operator<=(const char charArrayParam[], const String &stringParam);
//<= operator

bool operator<=(const unsigned char charArrayParam[], const String &stringParam);
//<= operator (for Pascal strings)

bool operator<=(char charParam, const String &stringParam);
//<= operator

//bool operator<=(unsigned char charParam, const String &stringParam);
//<= operator

const String operator+(const char charArrayParam[], const String &stringParam);
//+operator

const String operator+(const unsigned char charArrayParam[], const String &stringParam);
//+operator (for Pascal strings)

const String operator+(char charParam, const String &stringParam);
//+operator

//const String operator+(unsigned char charParam, const String &stringParam);
//+operator

ostream &operator<<(ostream &out, const String &outputMe);
//Outputs the passed string to the passed ostream

#endif
