#ifndef CLIENT_H
#define CLIENT_H

#include <iomanip.h>
#include <strstream.h>
#include <fstream.h>
#include <curses.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>

#include "bd_string.h"
#include "ptrlist.h"

#define BONIM Client::gClient()

enum ClientRunStatus
{
	crsRun=0,
	crsQuit=1,
	crsRestart=2
};

enum VisStatus
{
	visAll=0,
	visBuds=1
};

enum LowColDisp
//Yes, these start at 1 instead of 0, but it preserves backwards compatibility since the previous author used 1 and 2 in an int for some odd reason...
{
	lcdStatus=1,
	lcdTime=2
};

enum ScreenSize
{
	sszSmall=0,
	sszMedium=1,
	sszLarge=2
};

enum AvailStatus
{
	avsOffLine=0,
	avsOnLine=1,
	avsAway=2,
	avsIdle=3,
	avsNew=4	//Remove this one when real buddy list management is implemented
};

enum UserClass
{
	uclUnknown=0,
	uclTrial=1,
	uclAOL=4,
	uclFree=16
};

struct Buddy
{
	String budID;			//userarray[1]
	String budAlias; 		//userarray[0]
	AvailStatus budStatus;	//userarray[3]
	String budTime;			//userarray[2]
	UserClass budClass;		//userarray[4]
};

class Client
{
private:
	static Client *m_instance;
	
	PtrList<Buddy> m_buds;
	
	String m_version;
	
	String m_inputLine;
	String m_userID;
	String m_password;
	String m_target;
	String m_awayMsg;
	bool m_logIMs;
	VisStatus m_visStatus;
	bool m_beepOnMsg;
	bool m_showOffline;
	bool m_showAlias;
	bool m_showSigns;
	bool m_storePass;
	bool m_storePassFlagSet;
	LowColDisp m_lowColDisp;
	
	String m_lastFrom;
	String m_lastTo;
	
	ScreenSize m_screenSize;
	
	ClientRunStatus m_runStatus;
	
	Client();
	
public:
	String m_hitAndRunTarget;	/* Cheesy hack to get a feature a user really wanted. This like much of the code here will be fixed with better coding techniques. */
	String m_hitAndRunMessage;
	
	WINDOW *mainwin, *listwin, *inputwin, *statwin;
	
	~Client();
	
	static Client *gClient();
	
	bool addBud(const String &budID);
	
	bool setBudAlias(const String &budID, const String &budAlias);
	bool setBudClass(const String &budID, UserClass budClass);
	bool setBudStatus(const String &budID, AvailStatus budStatus);
	bool setBudTime(const String &budID, String &budTime);
	
	unsigned long getBudCount();
	
	Buddy getBudByPosition(unsigned long budPosition);
	Buddy getBudByID(const String &budID);
	Buddy getBudByAlias(String budAlias);
	
	bool killBud(String budID);
	void killBuds();
	
	const String &getVersion();
	
	void chopInputLine();
	void clearInputLine();
	void appendInputLine(char newChar);
	const String &getInputLine() const;
	
	bool setUserID(const String &newUserID);
	const String &getUserID() const;
	
	void setPassword(const String &newPassword);
	const String &getPassword() const;
	
	void setTarget(const String &newTarget);
	const String &getTarget() const;
	
	String getStatus() const;
	
	String getBonimDir() const;
	String getConfigDir() const;
	
	void setAwayMsg(const String &newAwayMsg);	//Set to "" to make yourself "unaway", but you should use unaway() to get feedback.
	const String &getAwayMsg() const;
	void unaway();
	
	void setLogIMs(bool newLogIMs);
	bool logIMs() const;
	
	void setVisStatus(VisStatus newVisStatus);
	VisStatus getVisStatus();
	
	void setBeepOnMsg(bool newBeepOnMsg);
	bool beepOnMsg();
	
	void setShowOffline(bool newShowOffline);
	bool showOffline();
	
	void setShowSigns(bool newShowSigns);
	bool showSigns();
	
	void setShowAlias(bool newShowAlias);
	bool showAlias();
	
	void setLowColDisp(LowColDisp newLowColDisp);
	LowColDisp getLowColDisp();
	
	void setLastFrom(const String &newLastFrom);
	String &getLastFrom();
	
	void setLastTo(const String &newLastTo);
	String &getLastTo();
	
	ScreenSize getScreenSize();
	
	void setRunStatus(ClientRunStatus newRunStatus);
	ClientRunStatus getRunStatus() const;
	
	void errUsage();
	void showPrompt();
	void displayStatus();
	void showUserList();
	void showNames();
	void clearAll();
	void displayHelp();
	
	void readConfig();
	void writeConfig();
	void makeConfig();
	
	int maincols;
	time_t htimeval;
	struct tm *thetime;
	String AOLUserList;
	
	void makeBudsOnlyVis(struct aim_conn_t *conn);	//These needs to be reworked, probably eliminated.
	int getBuddy(char *out, const char *in, int searchstatus, bool onlinestatus);
	void addUser();
	void delUser();
	
	long go(int args, char *argv[]);
	
private:
	Buddy *getBuddyPtrByID(String budID);
	
	String unformatBudID(String budID);
};

int faimtest_parse_oncoming(struct command_rx_struct *, ...);
int faimtest_parse_offgoing(struct command_rx_struct *, ...);
int faimtest_parse_login_phase3d_f(struct command_rx_struct *, ...);
int faimtest_auth_error(struct command_rx_struct *, ...);
int faimtest_auth_success(struct command_rx_struct *, ...);
int faimtest_parse_incoming_im(struct command_rx_struct *command, ...);
int faimtest_parse_userinfo(struct command_rx_struct *command, ...);
int faimtest_handleredirect(struct command_rx_struct *command, ...);
int faimtest_authsvrready(struct command_rx_struct *command, ...);
int faimtest_pwdchngdone(struct command_rx_struct *command, ...);
int faimtest_serverready(struct command_rx_struct *command, ...);
int faimtest_parse_misses(struct command_rx_struct *command, ...);

void refreshprofile(void);
void process_input(void);
void display_help_cmdprompt(void);
void send_reply(const char *cmd);
void process_command(const String &cmd);

#endif
