#ifndef PTRLISTNODE_TEMPLATE
#define PTRLISTNODE_TEMPLATE

//NOTE-These nodes are self linking at delete time
template <class PTRLISTNODETYPE>
class PtrListNode
{
private:
	PtrListNode* next;
	PtrListNode* previous;
	
	PTRLISTNODETYPE *data;
	
public:
	PtrListNode()
	//PURPOSE-Default constructor
	//POST-The node is constructed at default values
	{
		next=NULL;
		previous=NULL;
		
		data=NULL;
	}
	
	PtrListNode(PTRLISTNODETYPE *Data)
	//PURPOSE-Constructs the node with the data
	//IN-Data
	//POST-data=Data;
	{
		next=NULL;
		previous=NULL;
		
		data=Data;
	}
	
	PtrListNode(const PtrListNode &copyMe)
	//PURPOSE-Copy constructor
	//IN-copyMe
	//POST-The node is constructed with the data in copyMe (but not the list links!)
	{
		next=NULL;
		previous=NULL;
	
		*data=*(copyMe.data);
	}
		
	virtual ~PtrListNode()
	//PURPOSE-Destructor
	//POST-The node is destructed and it's neighbors are linked
	//	   to each other
	{
		if(next!=NULL) next->previous=previous;
		if(previous!=NULL) previous->next=next;
		
		if(data!=NULL) delete data;
	}

	PtrListNode *getNext()
	//PURPOSE-Get the next node
	//OUT-The next node
	//POST-The next node is returned
	{
		return next;
	}
	
	void setNext(PtrListNode *Next)
	//PURPOSE-Sets the next node
	//IN-Next
	//PRE-Make sure that the current next won't be leaked
	//POST-The next node becomes Next
	{
		next=Next;
	}
	
	PtrListNode *getPrevious()
	//PURPOSE-Get the previous node
	//OUT-The previous node
	//POST-The previous node is returned
	{
		return previous;
	}
	
	void setPrevious(PtrListNode *Previous)
	//PURPOSE-Sets the previous node
	//IN-Previous
	//PRE-Make sure that the current previous won't be leaked
	//POST-The next node becomes Next
	{
		previous=Previous;
	}
	
	PTRLISTNODETYPE *getData()
	//PURPOSE-Get the node's data
	//OUT-The node's data
	//POST-The node's data is returned
	{
		return data;
	}
	
	PTRLISTNODETYPE *releaseData()
	//PURPOSE-Gets the node's data and sets its pointer to the data to NULL
	//OUT-The node's data
	//POST-The node's data is returned and the node "forgets" the address of the data (but the data is not deallocated)
	{
		PTRLISTNODETYPE *tempData=data;
		data=NULL;
		return tempData;
	}
};

#endif

